import tkinter as tk
from tkinter import ttk
from FlightsRegister import FlightsRegister
from DirectFlight import DirectFlight
from MultiLegFlight import MultiLegFlight
from tkinter import messagebox as mbox
import os

fr = FlightsRegister()
multileg = False

def add_flight():
    if(multileg == False):
        flight = {
            "Flight Number": code_entry.get(),
            "Departure Airport": departureAirport_entry.get(),
            "Destination Airport": destinationAirport_entry.get(),
            "Duration in minutes": timeFlightInMinutes_entry.get()
            }
        try:
            fr.addNewFlight(flight["Flight Number"], flight["Departure Airport"], flight["Destination Airport"], int(flight["Duration in minutes"]))
            tree.insert('', 'end', values=(flight["Flight Number"], flight["Departure Airport"], flight["Destination Airport"], 1, flight["Duration in minutes"]))
            clear_entries()
        except Exception as e:
            mbox.showerror("Error", e)    
    else:
        flight = {
            "Flight Number": code_entry.get(),
            "Departure Airport": departureAirport_entry.get(),
            "Destination Airport": destinationAirport_entry.get(),
            "Duration in minutes": timeFlightInMinutes_entry.get()
            }
        airports = [departureAirport_entry.get()]
        for ap_textboxes in textboxes:
            airports.append(ap_textboxes.get())
        airports.append(destinationAirport_entry.get())
        time = []
        for t_textboxes in Dtextboxes:
            time.append(t_textboxes.get())
        time.append(timeFlightInMinutes_entry.get())
        #create legs list
        try:
            
            legs = []
            legs.append(DirectFlight(airports[0], airports[1], int(time[0])))
            i = 1
            while i <= len(airports)-2:
                legs.append(DirectFlight(airports[i], airports[i+1], int(time[i])))
                i+=1
            mlf = MultiLegFlight(legs)
            fr.addFlight(flight["Flight Number"], mlf)
            dep = MultiLegFlight.extractDepartureAirport(legs)
            dest = MultiLegFlight.extractDestinationAirport(legs)
            lg = mlf.getNumOfLegs()
            tm = mlf.getTimeFlightInMinutes()
            tree.insert('', 'end', values=(flight["Flight Number"], dep, dest, lg, tm))
            clear_entries()
        except Exception as e:
            mbox.showerror("Error", e)    
        
def codeList():
    codes = fr.getCodeFlightList()
    if codes:
        mbox.showinfo("Flight Code List: ", codes)
    else:
        mbox.showinfo("Code List Not Found", "No flights inserted in the flight register")             

def clear_entries():
    code_entry.delete('0', 'end')
    departureAirport_entry.delete('0', 'end')
    destinationAirport_entry.delete('0', 'end')
    timeFlightInMinutes_entry.delete('0', 'end')
    for ap in textboxes:
        ap.delete('0', 'end')
    for td in Dtextboxes:
        td.delete('0', 'end')    
    
def copyOn_file():
    fr.writeOnFile(str(textbox_fileName.get()))
    os.startfile(str(textbox_fileName.get()) + ".txt")
    textbox_fileName('0', 'end')

def searchFlight():
    code = str(textbox_code.get())
    found = fr.getFlight(code)
    if found:
        mbox.showinfo("Flight Found for Code: " + code, found.toString())
    else:
        mbox.showinfo("Flight Not Found", "No flight found for code: " + code)
    textbox_code.delete('0', 'end')

def toggle_textboxes():
    global multileg
    if switch_var.get() == 1:
        try:
            int(textbox_count.get())
        except ValueError as e:
            mbox.showerror("Error", "Number of legs not valid: \""+textbox_count.get()+"\"")
            switch_var.set(0)
            multileg = False
        if int(textbox_count.get()) <= 1:
            mbox.showerror("Error", "Number of legs not valid: \""+textbox_count.get()+"\"")
            switch_var.set(0)
            multileg = False
        else:
            multileg = True
        # Create leg_frame
        leg_frame.grid(row=1, column=1, padx=5, pady=5)
        # Create new textboxes
        num_textboxes = int(textbox_count.get())-1
        for i in range(num_textboxes):
            leg_label = tk.Label(leg_frame, text="Airport:")
            leg_label.grid(row=i+1, column=0, sticky="E")
            labels.append(leg_label)
            d_label = tk.Label(leg_frame, text="Duration (m):")
            d_label.grid(row=i+1, column=2, sticky="E")
            Dlabels.append(d_label)
            new_textbox = tk.Entry(leg_frame)
            new_textbox.grid(row=i+1, column=1, columnspan=1, padx=0, pady=5)
            textboxes.append(new_textbox)
            d_new_textbox = tk.Entry(leg_frame)
            d_new_textbox.grid(row=i+1, column=3, columnspan=1, padx=0, pady=5)
            Dtextboxes.append(d_new_textbox)
    else:
        # Remove textboxes and labels
        multileg = False
        for textbox in textboxes:
            textbox.grid_remove()
        textboxes.clear()
        for label in labels:
            label.grid_remove()
        labels.clear()
        for d_label in Dlabels:
            d_label.grid_remove()
        for d_textboxes in Dtextboxes:
            d_textboxes.grid_remove()
        Dlabels.clear()
        Dtextboxes.clear()
        # Hide leg_frame
        leg_frame.grid_remove()
        textbox_count.delete('0','end')
        

root = tk.Tk()
root.title("Flight Log")

# Creating the table to display flights
tree = ttk.Treeview(root, columns=("Flight Number", "Departure Airport", "Destination Airport", "Legs", "Duration in Minutes"))
tree.heading("#0", text="")
tree.heading("Flight Number", text="Flight Number")
tree.heading("Departure Airport", text="Departure Airport")
tree.heading("Destination Airport", text="Destination Airport")
tree.heading("Legs", text="Legs")
tree.heading("Duration in Minutes", text="Duration in Minutes")
tree.column("#0", width=0, stretch="no")
tree.column("Flight Number", width=100)
tree.column("Departure Airport", width=150)
tree.column("Destination Airport", width=150)
tree.column("Legs", width=50)
tree.column("Duration in Minutes", width=150)
tree.grid(row=0, columnspan=5, padx=10, pady=10)

# Label frame
label_frame = ttk.LabelFrame(root, text="Flight Details")
label_frame.grid(row=1, column=0, padx=5, pady=5)

# Leg frame
leg_frame = ttk.LabelFrame(root, text="Leg Airports")
leg_frame.grid(row=1, column=1, padx=5, pady=5)

# Labels and input fields for adding a new flight
code_label = tk.Label(label_frame, text="Flight Number:")
code_label.grid(row=1, column=0, sticky="E")
code_entry = tk.Entry(label_frame)
code_entry.grid(row=1, column=1, padx=5, pady=5)

departureAirport_label = tk.Label(label_frame, text="Departure Airport:")
departureAirport_label.grid(row=2, column=0, sticky="E")
departureAirport_entry = tk.Entry(label_frame)
departureAirport_entry.grid(row=2, column=1, padx=5, pady=5)

destinationAirport_label = tk.Label(label_frame, text="Destination Airport:")
destinationAirport_label.grid(row=3, column=0, sticky="E")
destinationAirport_entry = tk.Entry(label_frame)
destinationAirport_entry.grid(row=3, column=1, padx=5, pady=5)

timeFlightInMinutes_label = tk.Label(label_frame, text="Duration (m):")
timeFlightInMinutes_label.grid(row=4, column=0, sticky="E")
timeFlightInMinutes_entry = tk.Entry(label_frame)
timeFlightInMinutes_entry.grid(row=4, column=1, padx=5, pady=5)

# Button for adding a flight
add_button = tk.Button(root, text="Add Flight", command=add_flight)
add_button.grid(row=5, column=0, padx=0, pady=5)

# Button for claearing the textbox
add_button = tk.Button(root, text="     Clear     ", command=clear_entries)
add_button.grid(row=6, column=0, padx=0, pady=5)


# Switch variable
switch_var = tk.IntVar()

# Entry for specifying the number of textboxes
count_label = tk.Label(root, text="Legs:        ")
count_label.grid(row=6, column=2, columnspan=1, sticky="W")
textbox_count = tk.Entry(root, width=5)
textbox_count.grid(row=6, column=2, padx=0, pady=0, sticky="E")

# Switch
switch = tk.Checkbutton(root, text="Multi-Leg Flight", variable=switch_var, command=toggle_textboxes)
switch.grid(row=6, column=3, columnspan=1, padx=10, pady=10, sticky="W")

# Button to search a flight
find_button = tk.Button(root, text = "Search", command=searchFlight)
find_button.grid(row=7, column=0, columnspan=1, padx=0, pady=5)

# Button to print the list of flight codes
list_button = tk.Button(root, text = "Flight Code list", command=codeList)
list_button.grid(row=7, column=3, columnspan=1, padx=0, pady=5)

# Entry for specifying the code of the flight
code_label = tk.Label(root, text="Flight Code:")
code_label.grid(row=7, column=0, columnspan=1, sticky="E")
textbox_code = tk.Entry(root)
textbox_code.grid(row=7, column=1, columnspan=1, padx=0, pady=10)

# Button for write on file
write_button = tk.Button(root, text = "Save on file", command=copyOn_file)
write_button.grid(row=8, column=0, columnspan=1, padx=0, pady=5)

# Entry for specifying the name of the file
fileName_label = tk.Label(root, text="File Name:")
fileName_label.grid(row=8, column=0, columnspan=1, sticky="E")
textbox_fileName = tk.Entry(root)
textbox_fileName.grid(row=8, column=1, columnspan=1, padx=0, pady=10)

# List to store the dynamically created textboxes and labels
textboxes = []
Dtextboxes = []
labels = []
Dlabels = []

root.mainloop()