from Flight import Flight
from FlightNotValidException import FlightNotValidException

class MultiLegFlight(Flight):
    """Class that represents a multi-leg civil flight between n airports operated by the same aircraft"""
    def __init__(self, legs):
        self.legs = []
        a = MultiLegFlight.extractDepartureAirport(legs)
        b = MultiLegFlight.extractDestinationAirport(legs)
        Flight.__init__(self, a, b)
        if(len(legs) < 2):
            raise FlightNotValidException("Insufficient number of legs: "+str(len(legs)))
        i = 1
        while i < len(legs):
            if(not (legs[i-1].getDestinationAirport() == legs[i].getDepartureAirport())):
                raise FlightNotValidException("One Destination Airport doesn't match the Departure one of the next leg: "+legs[i].getDepartureAirport())
            i += 1
        self.legs = legs
        
    def extractDepartureAirport(legs):
        return legs[0].getDepartureAirport()
    
    def extractDestinationAirport(legs):
        return legs[len(legs)-1].getDestinationAirport()
    
    #@Override
    def getNumOfLegs(self):
        return len(self.legs)
    
    #@Override
    def getTimeFlightInMinutes(self):
        totTime = 0
        for flight in self.legs:
            totTime += flight.getTimeFlightInMinutes()
        return totTime
    
    #@Override
    def toString(self):
        return Flight.toString(self) + ", Number of Routes: "+str(self.getNumOfLegs())+", TimeFlight in Minutes: "+str(self.getTimeFlightInMinutes())