from FlightsRegister import FlightsRegister
from DirectFlight import DirectFlight
from MultiLegFlight import MultiLegFlight
from FlightNotValidException import FlightNotValidException
from FlightCodeNotValidException import FlightCodeNotValidException

import unittest

class tester(unittest.TestCase):
    
#-------------------------Direct Flight-----------------------------#
    def test_DirectFlight(self):
        flight = DirectFlight('LIN', 'FCO', 45)
        self.assertEqual('LIN', flight.getDepartureAirport())
        self.assertEqual('FCO', flight.getDestinationAirport())
        self.assertEqual(45, flight.getTimeFlightInMinutes())
    
    def test_ErrorDirectFlight(self):
        try:
            flight = DirectFlight('AAA', 'FCO', 45)
            self.fail()
        except FlightNotValidException as e:
            pass
        try:
            flight = DirectFlight('FCO', '', 45)
            self.fail()
        except FlightNotValidException as e:
            pass
        try:
            flight = DirectFlight('LIN', 'FCO', 0)
            self.fail()
        except FlightNotValidException as e:
            pass
    
    def test_DirectFlight_toString(self):
        flight = DirectFlight('LIN', 'FCO', 45)
        self.assertTrue(flight.toString().find("FCO") >= 0)
        self.assertTrue(flight.toString().find("LIN") >= 0)
        self.assertTrue(flight.toString().find("45") >= 0)
        self.assertTrue(flight.toString().find("1") >= 0)
    
#-----------------Multi Leg Flight---------------------------
    def test_MultiLegFlight(self):
        legs = []
        legs.append(DirectFlight('MXP', 'LIN', 20))
        legs.append(DirectFlight('LIN', 'FCO', 60))
        legs.append(DirectFlight('FCO', 'SFO', 665))
        
        mlf = MultiLegFlight(legs)
        self.assertEqual('SFO', MultiLegFlight.extractDestinationAirport(legs))
        self.assertEqual('MXP', MultiLegFlight.extractDepartureAirport(legs))
        self.assertEqual(745, mlf.getTimeFlightInMinutes())
        self.assertEqual(3, mlf.getNumOfLegs())
    
    def test_ErrorMultiLegFlight(self):
        f1 = DirectFlight('MXP', 'LIN', 60)
        f2 = DirectFlight('LIN', 'FCO', 60)
        legs = []
        legs.append(f2)
        legs.append(f1)
        try:
            mlf = MultiLegFlight(legs)
            self.fail()
        except FlightNotValidException as e:
            pass
        legs.clear()
        legs.append(f1)
        legs.append(f2)
        legs.append(f2)
        try:
            mlf = MultiLegFlight(legs)
            self.fail()
        except FlightNotValidException as e:
            pass
        
    def test_MultiLegFlight_toString(self):
        legs = []
        legs.append(DirectFlight('LIN', 'FCO', 60))
        legs.append(DirectFlight('FCO', 'SFO', 665))
        mlf = MultiLegFlight(legs)
        self.assertTrue(mlf.toString().find("SFO") >= 0)
        self.assertTrue(mlf.toString().find("LIN") >= 0)
        self.assertTrue(mlf.toString().find("725") >= 0)
        self.assertTrue(mlf.toString().find("2") >= 0)
        
#-----------------------Flight Register-----------------------------

    def test_Register_addFlight(self):
        df = DirectFlight('MXP', 'LIN', 20)
        legs = []
        legs.append(DirectFlight('MXP', 'LIN', 20))
        legs.append(DirectFlight('LIN', 'FCO', 60))
        legs.append(DirectFlight('FCO', 'SFO', 665))
        mlf = MultiLegFlight(legs)
        
        reg = FlightsRegister()
        try:
            reg.addFlight('AB1', mlf)
            self.fail()
        except FlightCodeNotValidException as e:
            pass
        reg.addFlight("P2A01", df)
        reg.addFlight("P2A02", mlf)
        
    def test_Register_addNewFlight(self):
        reg = FlightsRegister()
        try:
            reg.addNewFlight("P2A03", 'LIN', 'AAA', 660)
            self.fail()
        except FlightNotValidException as e:
            pass
        reg.addNewFlight("P2A03", 'LIN', 'FCO', 660)
        
    def test_Register_getCodeFlightList(self):
        df1 = DirectFlight('MXP', 'LIN', 20)
        reg = FlightsRegister()
        reg.addFlight("P2A01", df1)
        reg.addNewFlight("P2A03", 'LIN', 'FCO', 660)
        codes = reg.getCodeFlightList()
        self.assertEqual(2, len(codes))
        s = str(codes[0]) + str(codes[1])
        self.assertTrue(s.find("P2A01") >= 0)
        self.assertTrue(s.find("P2A03") >= 0)
        
    def test_Register_getFlight(self):
        df1 = DirectFlight('MXP', 'LIN', 20)
        reg = FlightsRegister()
        reg.addFlight("P2A01", df1)
        reg.addNewFlight("P2A03", 'LIN', 'FCO', 660)
        df2 = reg.getFlight("P2A03")
        self.assertEqual(660, df2.getTimeFlightInMinutes())
        df2 = reg.getFlight("P2A02")
        self.assertIsNone(df2)
        
    def test_Register_writeOnFlie(self):
        df1 = DirectFlight('MXP', 'LIN', 20)
        df2 = DirectFlight('FCO', 'SFO', 665)
        
        legs = []
        legs.append(DirectFlight('MXP', 'LIN', 20))
        legs.append(DirectFlight('LIN', 'FCO', 60))
        legs.append(DirectFlight('FCO', 'SFO', 665))
        mlf = MultiLegFlight(legs)
        
        reg = FlightsRegister()
        reg.addFlight("P2A01", df1)
        reg.addFlight("P2A08", mlf)
        reg.addFlight("P2A03", df2)
        
        fileName = "flightList.txt"
        reg.writeOnFile(fileName)
        
        f = open(fileName, 'r')
        data = f.read()
        f.close()
        self.assertTrue(data.find("FCO") >0)
        self.assertTrue(data.find("745") >0)
        self.assertTrue(data.find("665") >0)
     
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(tester)
    unittest.TextTestRunner(verbosity=2).run(suite)